/****************** AHB Interface PSL Sugar Assertions ****************/
`timescale 1 ns / 100 ps
module ahbCompliance (/*AUTOARG*/
   // Inputs
   hrdata, 
   hclk, hresetn, haddr, htrans, hwrite, hsize, hburst, hprot, 
   hwdata, hsel, hready, hresp
   );
//  `define SIM 1
`include "ahb_param.v"
	 
   input hclk;        
   input hresetn;     
   input [31:0] haddr; 
   input [htrans_width -1:0] htrans; 
   input 		     hwrite;      
   input [hsize_width -1:0]  hsize;  
   input [hburst_width -1:0] hburst; 
   input [3:0] 		     hprot;  
   input [31:0] 	     hwdata;
   input 		     hsel;       
   input 		     hready;      
   input [hresp_width -1:0]  hresp;

   input [31:0] 	     hrdata;
   

   // Transfer response: OKAY, ERROR, RETRY and SPLIT.

   parameter 		     dataBusWidth = 32;
   parameter 		     numSlaves = 32;
   parameter 		     busMaster = 0;
   
   reg [31:0] 		     prev_haddr, prev_haddrPlusSize;
   reg 			     prev_hwrite, prev_hready;
   reg [numSlaves-1:0] 	     prev_hsel;
   reg [3:0] 		     prev_hprot;
   reg [hsize_width-1:0]  prev_hsize;
   reg [hburst_width-1:0] prev_hburst;
   reg [dataBusWidth-1:0]    prev_hrdata, prev_hwdata;
   reg [htrans_width-1:0] 		     prev_htrans, prev_hresp;
   integer 		     AddrIncrement;
   integer 		     NumberBeats=0;
   /* capture data that is needed in the following assertions */
   always @(posedge hclk or negedge hresetn)
     begin
	prev_haddr <= haddr;
	prev_hready <= hready;
	prev_hwrite <= hwrite;
	prev_hrdata <= hrdata;
	prev_hwdata <= hwdata;
	prev_htrans <= htrans;
	prev_hburst <= hburst;
	prev_hresp <= hresp;
	prev_hsel <= hsel;
	prev_hsize <= hsize;
	prev_hprot <= hprot;
	case (hsize) /* AddrIncrement is used to check the address */
	  bits8: AddrIncrement=1;
	  bits16: AddrIncrement=2;
	  bits32: AddrIncrement=4;
	  bits64: AddrIncrement=8;
	  bits128: AddrIncrement=16;
	  bits256: AddrIncrement=32;
	  bits512: AddrIncrement=64;
	  bits1024: AddrIncrement=128;
	  default: ;
	endcase
	/* NumberBeats is used to check packet lengths */
	if (htrans == IDLE)
	  NumberBeats <= 0;
	else if ( htrans == NONSEQ)
	  NumberBeats <= 1;
	else if ( (htrans == SEQ) && (hready == 1) )
	  NumberBeats <= NumberBeats + 1;
	else
	  NumberBeats <= NumberBeats;
	/* determine the predicted address */
	if ((htrans != BUSY) && (hready != 0))
	  prev_haddrPlusSize <= haddr + AddrIncrement;
     end
   /**************************** PSL ASSERTIONS **************************/
   /* define default clock that will be used to sample all subsequent asserts */
   /* NOTE: it is important that data does not change on the same edge as clock
		because, like HDL, assertions can be sensitive to races */
   // psl default clock = (posedge hclk);
   // --------------------------------------------------
   /* Behavior: If reset is active then Transfer Type is IDLE and Resp is OKAY */
   // psl property IdleAndOkayAfterReset = always(
   // {(hresetn == 0);(hresetn == 1)} |->
   // {(htrans == IDLE) && (hresp == OKAY)});
   // psl assert IdleAndOkayAfterReset;
   //
   // --------------------------------------------------   
   /* Behavior: If the transfer type is NONSEQ and Burst Mode is SINGLE,
		then on the next clock cycle, the transfer type is not SEQ and not BUSY */
   // psl property BusyAndSeqNeverFollowNonseqOnSingleTransfer = always (
   // (htrans == NONSEQ) && (hburst == SINGLE ) ->
   // next ((htrans != SEQ) && (htrans != BUSY)) );
   // psl assert BusyAndSeqNeverFollowNonseqOnSingleTransfer;
   //
   // --------------------------------------------------
   /* Behavior: If the transfer type is BUSY and slave is ready,
		then on the next clock cycle, the transfer type must not be IDLE and
		must not be NONSEQ unless grant is 0 and ready is 1  */
   // psl property NeverGoFromBusyToIdleOrNonseqUnlessNoGrant = always (
   // ((htrans == BUSY) && (hready == 1)) ->
   // next ((htrans != IDLE) && (htrans != NONSEQ)));
   //// abort ((hGnt == 0) && (hready == 1)) || (hresetn == 0));
   // psl assert NeverGoFromBusyToIdleOrNonseqUnlessNoGrant; 
   //
   // --------------------------------------------------  
   /* Behavior: if the transfer type is IDLE, then on the next clock
		cycle, the transfer type must be either IDLE or NONSEQ (default clock)*/
   // psl property AlwaysGoToNonseqFromIdle = always (
   // (htrans == IDLE) ->
   // next ((htrans == IDLE) || (htrans == NONSEQ)) );
   // psl assert AlwaysGoToNonseqFromIdle; 
   //%% from code review BD 10/11/03: 
   //%% "The property is a duplicate of NeverGoFromIdleToSeqOrBusy below."
   //
   // --------------------------------------------------    
   /* Behavior: if the transfer type is BUSY then on the corresponding
		data transfer , the slave must provide a zero wait state OKAY response */
   // psl property ResponseToBusyMustBeZeroWaitOKAY = always (
   // ((htrans == BUSY) && (hready == 1)) ->
   // next ((hresp == OKAY) && (hready == 1)) );
   // psl assert ResponseToBusyMustBeZeroWaitOKAY; 
   //
   // --------------------------------------------------      
   /* Behavior: if the transfer type is IDLE then on the corresponding data
		transfer, the slave must provide a zero wait state OKAY response */
   // psl property ResponseToIdleMustBeZeroWaitOKAY = always (
   // ((htrans == IDLE) && (hready == 1)) ->
   // next ((hresp == OKAY) && (hready == 1)) );
   // psl assert ResponseToIdleMustBeZeroWaitOKAY; 
   //
   // --------------------------------------------------      
   /* Behavior: if the previous response was OKAY, and the current response
		is not OKAY, then Ready must be 0 during the second not OKAY response */
   // psl property ReadyMustBeZeroDuringFirstNotOkayResponse = always (
   // {(hresp == OKAY);(hresp != OKAY)} |->
   // {hready == 0}) ;
   // psl assert ReadyMustBeZeroDuringFirstNotOkayResponse;
   //
   //%% From code review BD 10/11/03:
   //%% I can t find a check anywhere that hready must be true in the following cycle. 
   //%% It s a requirement of the protocol that
   //%% ((hresp != OKAY) && !hready) -> next ((hresp != OKAY) && hready)
   //%% It s also a requirement that hresp be unchanged in these two cycles
   //%% And this property as written, while correct, fails to handle the condition 
   //%% where the data phase is zero wait state
   // psl property HrespUnchangedWhenNewHready=
   //   always (((hresp != OKAY) && !hready) -> next ((hresp != OKAY) && hready));
   // psl assert HrespUnchangedWhenNewHready; 
   //
   // --------------------------------------------------      
   // Behavior: if the response is SPLIT or RETRY, the master must drive IDLE
   //// psl property FirstNotOkayResponseCausesNextIdle = always (
   //// ((hresp == SPLIT) || (hresp == RETRY)) && (hready == 0) ->
   //// next (htrans == IDLE));
   //%% From Code review BD 10/11/03:
   //%% No, this is absolutely NOT a requirement. It s not true when the 
   //%% current address transfer and current data transfer are from different masters, 
   //%% i.e., when bus is in the process of changing mastership
   //
     // --------------------------------------------------    
   /* Behavior: if the transfer type is SEQ or BUSY, then the controls of the
		corresponding data transfer must be the same as their previous control
		NOTE: the equivalence operator is used with hprot
		because this is an optional signal and may be an "X" value if not
		connected */
   // psl property ControlMustBeConstantDuringABurst = always (
   // (htrans == SEQ) || (htrans == BUSY) ->
   // ((hsize == prev_hsize) && (hprot === prev_hprot) &&
   // (hwrite == prev_hwrite)) );
   // psl assert ControlMustBeConstantDuringABurst; 
   //
   //%% From code review BD 10/11/03:
   //%% It s confusing to talk about the controls of the corresponding 
   //%% data transfer.  Tthe use of the implication means the property is comparing the 
   //%% controls for the CURRENT address with the previous CYCLE. Note this is not
   //%% the previous address TRANSFER.
   //%% In general, it is recommended to use properties that do not rely on 
   //%% latches for storage, rather than the expressiveness of the language. 
   //%% The requirement could be re-written 100% in PSL without using the latches as:
   //%% forall i in boolean:
   //%%   forall j in {bits8:bits1024 }:
   //%%     forall k in {0:15 } :
   //%%        forall h in {INCR:INCR16} :
   //%%          always ({(hready && (htrans==NONSEQ) && (hburst==h) 
   //%%                   && (hwrite==i) && (hsize==j) &&(hprot==k))} |=> 
   //%%                      {((hburst==h) && (hwrite==i) && (hsize==j) 
   //%%                          &&(hprot==k) && 
   //%%                          ((htrans==SEQ)|| (htrans==BUSY)))[*];
   //%%                          ((htrans!=SEQ)&&(htrans!=BUSY)) } );
   //%% which is actually a more comprehensive property because it checks 
   //%% hburst is constant, which is omitted from the original
   //
   // --------------------------------------------------  
   /* Behavior: if Ready is 0 and the response is OK or ERROR, the address
		and control, and data from the master are held constant (default clock)
		NOTE: the equivalence operator is used with hprot because this is an
		optional signal and may be an "X" value if not connected */
   // psl property ControlMustBeConstantWhenSlaveNotReady = always (
   // (hready == 0) && ((hresp == OKAY) || (hresp == ERROR)) -> next
   // (htrans == prev_htrans) && (hsize == prev_hsize) &&
   // (hprot === prev_hprot) && (hwrite == prev_hwrite) &&
   // (haddr == prev_haddr) && (hburst == prev_hburst) 
   // abort (hresetn == 0));
   // psl assert ControlMustBeConstantWhenSlaveNotReady;
   //
   //%% From code review BD 10/11/03:
   //%% The comment mentions data but it s not included in the SERE. 
   //%% The requirement also holds for transfers that end in
   //%% SPLIT & RETRY up to the cycle where hready is low, 
   //%% and HRESP != OKAY, but this doesn t seem to be checked anywhere
   //
   // --------------------------------------------------    
   /* Behavior: if ready is 0 and the previous transfer type was SEQ or NONSEQ,
		then on the next cycle the write data is the same as the previous write data
		(default clock) (This assertion is designed to fail!!!) */
   // psl property WriteDataMustBeConstantWhenSlaveNotReadyAndDataBeingTransferred = always (
   // (hready == 0) && (hwrite == 1) &&
   // ((prev_htrans == SEQ) || (prev_htrans == NONSEQ)) -> next
   // (hwdata == prev_hwdata) 
   // abort (hresetn == 0));
   // psl assert WriteDataMustBeConstantWhenSlaveNotReadyAndDataBeingTransferred;
   //
   //%% From code review BD 10/11/03
   //%% IMHO this is not a good way to demonstrate a failing assertion. 
   //%% The logic of the property is completely confused
   //%% because the pipelined address architecture does not allow you 
   //%% to reason about hwrite and hwdata in this way. It s
   //%% very difficult to show how to make small changes to this property to make it correct.
   //
   // --------------------------------------------------    
   /* Behavior: if transfer type is BUSY then address does not change  */
   // psl property AddrHeldWhenMasterBusy = always (
   // (htrans == BUSY) -> next (haddr == prev_haddr) );
   // psl assert AddrHeldWhenMasterBusy; 
   //
   // --------------------------------------------------    
   /* Behavior: if transfer is in progress, then 1K boundary is not exceeded  */
   // psl property PageAddressNeverExceeds1kBoundary = always (
   // ((htrans == SEQ) || (htrans == BUSY)) ->
   // (haddr[31:10] == prev_haddr[31:10]) );
   // psl assert PageAddressNeverExceeds1kBoundary;
   //
   //%% From code review BD 10/11/03:
   //%% This also applies to the first/second transfer in the burst. 
   //%% They must also be in the same 1k address page.
   //
   // --------------------------------------------------    
   /* Behavior: If the burst type is not INCR and the transfer type is not
		IDLE , then the NumberBeats for the Burst Mode is not exceeded  */
   // psl property BurstIsNotTooLong = always (
   // (hburst != INCR) && (htrans != IDLE) ->
   // ((hburst == SINGLE) && (NumberBeats <= 1)) ||
   // ((hburst == WRAP4) && (NumberBeats <= 4)) ||
   // ((hburst == WRAP8) && (NumberBeats <= 8)) ||
   // ((hburst == WRAP16) && (NumberBeats <= 16)) ||
   // ((hburst == INCR4) && (NumberBeats <= 4)) ||
   // ((hburst == INCR8) && (NumberBeats <= 8)) ||
   // ((hburst == INCR16) && (NumberBeats <= 16)) );
   // psl assert BurstIsNotTooLong;
   //
   //%% From code review BD 10/11/03:
   //%% Once again, this does not exploit the expressiveness of PSL, 
   //%% and I believe it can be re-written 100% in PSL as three
   //%% or four alternative outcomes without needing the  forall 
   //%% construct, and without requiring the beat counting logic.
   //
   // --------------------------------------------------     
   /* Behavior: For all but INCR Burst mode, if the end of the packet is being
		transferred as indicated by a transition from SEQ to IDLE when Resp is ok
		then the NumberBeats for the Burst Mode is the max number unless grant is 0 */
   // 7/16/03 Modified to correct error in number because of last increment
   // psl property BurstIsNotTooShort = always (
   // {((hburst != INCR) && (hresp == OKAY) && (htrans == SEQ));
   // (htrans == IDLE) } |->
   // {((prev_hburst == SINGLE) && (NumberBeats == 1)) ||
   // ((prev_hburst == WRAP4) && (NumberBeats == 4+1)) ||
   // ((prev_hburst == WRAP8) && (NumberBeats == 8)) ||
   // ((prev_hburst == WRAP16) && (NumberBeats == 16)) ||
   // ((prev_hburst == INCR4) && (NumberBeats == 4)) ||
   // ((prev_hburst == INCR8) && (NumberBeats == 8)) ||
   // ((prev_hburst == INCR16) && (NumberBeats == 16))});
   //// abort (hGnt == 0)) ;
   // psl assert BurstIsNotTooShort;
   //
   //%% From code review BD 10/11/03
   //%% Ditto the previous comment.
   //%% In addition this looks VERY suspicious. Why is the WRAP4 beat 
   //%% count 4+1? Also it doesn t take care of the situation
   //%% where you have back-to-back operations  a burst may be followed by a NONSEQ
   //
   // --------------------------------------------------    
   /* Behavior: if the transfer type is IDLE , then on the next clock cycle,
		the transfer type is not SEQ and the transfer type is not BUSY  */
   // psl property NeverGoFromIdleToSeqOrBusy = always (
   // (htrans == IDLE) ->
   //   next ((htrans != SEQ) && (htrans != BUSY)) );
   // psl assert NeverGoFromIdleToSeqOrBusy;
   // 
   //%% From code review BD 10/11/03
   //%% The property is a duplicate of AlwaysGoToNonseqFromIdle above.
   //
   // --------------------------------------------------    
   /* Behavior: RETRY is always asserted for two cycles unless reset is asserted */
   // psl property RetryResponseMustPersistTwoCycles = always (
   //   {(hresp != RETRY);(hresp == RETRY)} |=> 
   //                         {(hresp == RETRY)} abort (hresetn == 0));
   // psl assert RetryResponseMustPersistTwoCycles;
   //
   //%% From code review 10/11/03
   //%% This doesn t fully verify back-to-back operations without wait states.
   //
   // --------------------------------------------------    
   /* Behavior: SPLIT is always asserted for two cycles unless reset is asserted */
   // psl property SplitResponseMustPersistTwoCycles = always (
   //   {(hresp != SPLIT);(hresp == SPLIT)} |=>
   //                {(hresp == SPLIT)} abort (hresetn == 0));
   // psl assert SplitResponseMustPersistTwoCycles;
   //
   //%% From code review 10/11/03
   //%% This doesn t fully verify back-to-back operations without wait states.
   //
   // --------------------------------------------------    
   /* Behavior: ERROR is always asserted for two cycles unless reset is asserted */
   // psl property ErrorResponseMustPersistTwoCycles = always (
   //   {(hresp != ERROR);(hresp == ERROR)} |=>
   //         {(hresp == ERROR)} abort (hresetn == 0));
   // psl assert ErrorResponseMustPersistTwoCycles;
   //
   //%% From code review 10/11/03
   //%% This doesn t fully verify back-to-back operations without wait states.
   //
   // --------------------------------------------------    
   /******* incorrectAddressCalculation for 3 example WRAP modes: This calculates
		whether the next address is correct for all wrap modes*******/
   
   /* Behavior: if Burst mode is WRAP4 and Size is 8 bits, then if
		transfer mode is SEQ or BUSY, then if previous transfer mode is not
		busy and previous Ready is not zero, then check that the address is as predicted */
   // psl property CorrectAddressDuringPageSize4BurstWrap = always (
   // ((hburst== WRAP4) && (hsize == bits8)) ->
   // ((htrans == SEQ || htrans == BUSY)) ->
   // ((prev_htrans != BUSY) && (prev_hready != 0)) ->
   // ((haddr[31:2] == prev_haddr[31:2]) &&
   // (haddr[1:0] == prev_haddrPlusSize[1:0])) );
   // psl assert CorrectAddressDuringPageSize4BurstWrap;
   //
   // --------------------------------------------------    
   // psl property CorrectAddressDuringPageSize8BurstWrap = always (
   // ((hburst== WRAP4) && (hsize == bits16)) ||
   // ((hburst== WRAP8) && (hsize == bits8)) ->
   // ((htrans == SEQ) || (htrans == BUSY)) ->
   // ((prev_htrans != BUSY) && (prev_hready != 0)) ->
   // ((haddr[31:3] == prev_haddr[31:3]) &&
   // (haddr[2:0] == prev_haddrPlusSize[2:0])) );
   // psl assert CorrectAddressDuringPageSize8BurstWrap;
   //
   // --------------------------------------------------  
   // psl property CorrectAddressDuringPageSize2048BurstWrap = always (
   // ((hburst== WRAP16) && (hsize == bits1024)) ->
   // ((htrans == SEQ) || (htrans == BUSY)) ->
   // ((prev_htrans != BUSY) && (prev_hready != 0)) ->
   // ((haddr[31:11] == prev_haddr[31:11]) &&
   // (haddr[10:0] == prev_haddrPlusSize[10:0])) );
   // psl assert CorrectAddressDuringPageSize2048BurstWrap;
   //
   // --------------------------------------------------    
   /* Behavior: the address must increment by size during all INCR beats */
   // psl property AddressIncBySizeDuringAllBurstIncrBeats = always (
   // ((hburst == INCR16) || (hburst == INCR8) ||
   // (hburst == INCR4)) ->
   // ((htrans == SEQ) || (htrans == BUSY)) ->
   // ((prev_htrans != BUSY) && (hready != 0)) ->
   // (haddr == prev_haddrPlusSize) );
   // psl assert AddressIncBySizeDuringAllBurstIncrBeats;
   //
   // --------------------------------------------------     
   /* Behavior: always address must be aligned to the transfer size */
   // psl property AddressNotAlignedToTransferSize = always (
   // ((hsize == bits8) ||
   // ((hsize == bits16) && (haddr[0] == 1'b0)) ||
   // ((hsize == bits32) && (haddr[1:0] == 2'b00)) ||
   // ((hsize == bits64) && (haddr[2:0] == 3'b000)) ||
   // ((hsize == bits128) && (haddr[3:0] == 4'b0000)) ||
   // ((hsize == bits256) && (haddr[4:0] == 5'b00000)) ||
   // ((hsize == bits512) && (haddr[5:0] == 6'b000000)) ||
   // ((hsize == bits1024) && (haddr[6:0] == 7'b0000000)) ) );
   // psl assert AddressNotAlignedToTransferSize;
   //
   // --------------------------------------------------    
   /* Behavior: always the maximum number of wait states is 16 */
   // psl property NeverMoreThan16WaitStates = always (
   // {(hready == 1);(hready == 0)} |=>
   // {{(hready == 0)[*0..15]};{hready == 1}}
   // abort (hresp != OKAY));
   // psl assert NeverMoreThan16WaitStates;
   //
   //%% From code review BD 10/11/03
   //%% This is strictly a recommendation, not a requirement, 
   //%% though if all slaves are designed to meet the recommendation
   //%% it could represent a failure in most systems. 
   //%% The coding is very crude, with the use of the abort, which can be
   //%% avoided with something like:
endmodule // ahbCompliance

