//                              -*- Mode: Verilog -*-
// Filename        : server.v
// Description     : Server
// Author          : ben cohen
// Created On      : Fri Jul 18 17:23:29 2003
// Last Modified By: Ben 
// Last Modified On: .
// Update Count    : 0
// Status          :
// Date        Version  Author  Description
// 2003-06-20  1.0      ben2	Created
// 2003-07-18  2.0      ben2    change porality of hwrite

`timescale 1 ns / 100 ps
// ************************************************************
//****************** The Task (Transaction command) *****
/* -----\/----- EXCLUDED -----\/-----
`define IDLE         0
`define RESET        1
`define READ_SINGLE  2
`define READ_SEQ4    3
`define WRITE_SINGLE 4
`define WRITE_1OFN   5
`define WRITE_NOFM   6
`define WRITE_LASTOFM 7
`define WRITE_WRAP4  8
`define WRITE_WRAP8  9
 -----/\----- EXCLUDED -----/\----- */

	module thetask ();
  
	 parameter SIZE = 32;
   integer 	 job;
   reg [SIZE-1:0] data;	// data to load
   reg [SIZE-1:0] addr;	// starting address
endmodule // thetask

//******************** SERVER ********************
module server   (
								 hrdata,
								 hready,
								 hresp,
								 hwdata,
								 haddr,
								 htrans,
								 hwrite,
								 hsize,
								 hburst,
								 hprot,
								 hsel_sn,
								 hresetn,
								 hclk
								 ) ;
  `include "ahb_param.v"
  `include "localparam.v"  

	 input [31:0] hrdata;   
	 input 				hready;  
	 input [hresp_width -1:0] 	hresp;  
	 output [31:0] hwdata;
	 output [31:0] haddr; 
	 output [htrans_width -1:0]  htrans;
	 output 			 hwrite;
	 output [hsize_width -1:0]  hsize; 
	 output [hburst_width -1:0]  hburst;
	 output [3:0]  hprot; 
	 output 			 hsel_sn;
	 output 			 hresetn;
	 output 			 hclk   ;
	 
	 reg [31:0] 	 prev_hwdata; // 7/15 to maintain previous data
	 reg [31:0] 	 hwdata;
	 reg [31:0] 	 haddr; 
	 reg [htrans_width -1:0] 		 htrans;
	 reg 					 hwrite;
	 reg [hsize_width -1:0] 		 hsize; 
	 reg [hburst_width -1:0] 		 hburst;
	 reg [3:0] 		 hprot; 
	 reg 					 hsel_sn;
	 reg 					 hresetn;
	 reg 					 hclk   ;

   always @ (posedge hclk ) 
		 begin
				#2 prev_hwdata <= hwdata;
		 end
  
   // Clock 
   initial 
		 begin
				hclk = 1'b1;
				hresetn = 1'b1;
				forever #30 hclk = ~hclk;
		 end
  
   always @ (client.xactn) 
		 begin
				case (thetask.job)
					T_IDLE       : idle;
					RESET        : reset;
					READ_SINGLE  : read1;
					READ_SEQ4    : read_seq4;
					WRITE_SINGLE : write1;
					WRITE_1OFN   : write1ofn;
					WRITE_NOFM   : writenofm;
					WRITE_LASTOFM : writelastofm;
					//WRITE_WRAP4   : 
					//WRITE_WRAP8  9
					//WRITE_WRAP4  : write4;
					//WRITE_WRAP8  : write8;
					default       : $display($time, "Error in client transaction");
				endcase // case(job)
				-> client.done;	
		 end // always @ (client.xactn)
	 
   task reset;
			begin
				 #1;
				 hwdata =  32'bx;
				 haddr =   32'bx;
				 htrans = IDLE;
				 hwrite =  1'b0;
				 hsize =   bits32;
				 hburst =  SINGLE;
				 //	   hprot =   
				 hsel_sn = 1'b0;  
				 hresetn = 1'b1;
				 @ (posedge hclk);
				 #1 	  hresetn = 1'b0;
				 @ (posedge hclk);
				 #1      hresetn = 1'b1;
				 @ (posedge hclk);
			end
   endtask // idle
  
  
   task idle;
			begin
				 #1;
				 hwdata =  prev_hwdata;
				 haddr =   32'h55555550;
				 htrans = IDLE;
				 hwrite =  1'b0;
				 hsize =   bits32;
				 hburst =  SINGLE;
				 //	   hprot =   
				 hsel_sn = 1'b0;  // <- worst case test
				 hresetn = 1'b1;
				 @ (posedge hclk);
			end
   endtask // idle

   // 7/15/03  from ControlMustBeConstantWhenSlaveNotReady
   //   TB needs to be modified to comply to that requirement   
   task read1;
			begin
				 #1;
				 hwdata =  prev_hwdata;
				 haddr =   thetask.addr;
				 htrans = NONSEQ;
				 hwrite =  1'b0;
				 hsize =   bits32;
				 hburst =  SINGLE;
				 //	   hprot =   
				 hsel_sn = 1'b0;  // <- worst case test
				 hresetn = 1'b1;
				 @ (posedge hclk);
				 # 2;
				 // htrans = SEQ; // 7/15 <- error
				 htrans = IDLE;
				 // 7/15/03
				 #2;
				 while (hready==1'b0) begin
						// keep same signals 
						@ (posedge hclk);
						#2;
				 end
				 // fix issue brought about by 
				 // tb.ahbCompliance.WriteDataMustBeConstantWhenSlaveNotReadyAndDataBeingTransferred 
				 @ (posedge hclk); // 7/15 
			end
   endtask // read1
	 
   // 7/15/03  from ControlMustBeConstantWhenSlaveNotReady
   //   TB needs to be modified to comply to that requirement 
   task read_seq4;
			begin: Read4
				 integer i;
				 reg [31:0] temp_addr;
				 temp_addr = thetask.addr;
				 #1;
				 i=0;
				 hwdata =  prev_hwdata;
				 haddr =   temp_addr;
				 htrans = NONSEQ;
				 hwrite =  1'b0;
				 hsize =   bits32;
				 hburst =  WRAP4;
				 //	   hprot =   
				 hsel_sn = 1'b0;  // <- worst case test
				 hresetn = 1'b1;
				 @ (posedge hclk);
				 // 2,3, and 4th word read
				 #2; 	  htrans = SEQ;
				 while (i <= 3) begin
						#2;
						if (hready==1'b0) begin
							 //hwdata =  32'hxxxxxxxx;
							 //haddr =   32'hxxxxxxx0;
							 //htrans = SEQ;
							 //hwrite =  1'b0;
							 //hsize =   bits32;
							 //hburst =  WRAP4;
							 //	   hprot =   
							 //hsel_sn = 1'b0;  // <- worst case test
							 //hresetn = 1'b1;
							 @ (posedge hclk);
						end
						else  begin  // hready==1'b1
							 i=i+1;
							 //tb.ahbCompliance.ControlMustBeConstantWhenSlaveNotReady
							 // Thus, address must not change 
							 // temp_addr=temp_addr + 4*i; 
							 hwdata =  prev_hwdata;
							 haddr =   temp_addr;
							 htrans = SEQ; 
							 hwrite =  1'b0;
							 hsize =   bits32;
							 hburst =  WRAP4;
							 //	   hprot =   
							 hsel_sn = 1'b0;  // <- worst case test
							 hresetn = 1'b1;
							 @ (posedge hclk);
						end
				 end
				 #2 htrans = IDLE;
				 // 7/15/03
				 @ (posedge hclk);
			end
   endtask // read1
      
   task write1;
			begin
				 #1;
				 hwdata =  prev_hwdata;
				 haddr =   thetask.addr;
				 htrans = NONSEQ;
				 hwrite =  1'b1;
				 hsize =   bits32;
				 hburst =  SINGLE;
				 //	   hprot =   
				 hsel_sn = 1'b0;  
				 hresetn = 1'b1;
				 @ (posedge hclk);
				 #1;
				 hwdata =  thetask.data;
				 haddr =   thetask.addr;
				 htrans = IDLE;
				 hwrite =  1'b1;
				 hsize =   bits32;
				 hburst =  SINGLE;
				 //	   hprot =   
				 hsel_sn = 1'b0;  
				 hresetn = 1'b1;
				 @ (posedge hclk);
			end
   endtask // write1
	 
	 //7-15
   task write1ofn;
			begin
				 #1;
				 hwdata =  prev_hwdata;
				 haddr =   thetask.addr; // 1st word of multiple writes 
				 htrans = NONSEQ;
				 hwrite =  1'b1;
				 hsize =   bits32;
				 hburst =  SINGLE;
				 //	   hprot =   
				 hsel_sn = 1'b0;  
				 hresetn = 1'b1;
				 @ (posedge hclk);
			end
   endtask // write1ofn
	 
   task writenofm;
			begin
				 #1;
				 hwdata =  thetask.data; // nth-1 word of write 
				 haddr =   thetask.addr; // nth address 
				 htrans = NONSEQ;
				 hwrite =  1'b1;
				 hsize =   bits32;
				 hburst =  SINGLE;
				 //	   hprot =   
				 hsel_sn = 1'b0;  
				 hresetn = 1'b1;
				 @ (posedge hclk);
			end
   endtask // writenofm
	 
   task writelastofm;
			begin
				 #1;
				 hwdata =  thetask.data; // nth-1 word of write 
				 haddr =   thetask.addr; // nth address 
				 htrans = IDLE;
				 hwrite =  1'b1;
				 hsize =   bits32;
				 hburst =  SINGLE;
				 //	   hprot =   
				 hsel_sn = 1'b0;  
				 hresetn = 1'b1;
				 @ (posedge hclk);
			end
   endtask // writelastofm
endmodule // server
